/*   cb2Bib References CSS   */

/*   general   */

body {
font-size: 100%;
font-family: sans-serif;
background-color: #818B93;
color: black;
padding: 0;
margin: 0;
}

a {
color: #D8D7D7;
text-decoration: none;
}

a:link {
color: #D8D7D7;
text-decoration: underline;
}

h1
{
color: #C7C6C6;
font-size: 20px;
font-variant: small-caps;
font-family: sans-serif;
}


/*   references   */

#references{
background-color: #818B93;
}

#references blockquote{
font-size: small;
}

#references span{
color: #E4E3E2;
}

#excerpt{
font-style: italic;
}

/*   header   */

#header {
vertical-align: middle;
}

/*   footer   */

#footer {
color: #C7C6C6;
font-family: arial, sans-serif;
font-size: small;
}

#footer td{
padding: 10px;
padding-left: 5px;
padding-right: 5px;
}
